"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const basic_1 = require("./basic");
const constants_1 = require("../constants");
const complex_1 = require("./complex");
const text_1 = require("./text");
const utils_1 = require("../utils");
/**
 * Returns a protocol
 *
 * @example
 *
 * protocol()
 * // => 'http'
 */
function protocol() {
    return complex_1.oneOf(constants_1.PROTOCOLS.split(' '));
}
exports.protocol = protocol;
/**
 * Returns a tld
 *
 * @example
 *
 * tld()
 * // => 'hk'
 */
function tld() {
    return complex_1.oneOf(constants_1.TLDS.split(' '));
}
exports.tld = tld;
/**
 * Returns an ip address
 *
 * @example
 *
 * ip()
 * // => '233.173.228.16
 */
function ip() {
    return Array
        .from({ length: 4 })
        .map(() => basic_1.integer(0, 255))
        .join('.');
}
exports.ip = ip;
/**
 * Returns an ipv6 address
 *
 * @example
 *
 * ipv6()
 * // => 'b35c:9ff1:66d4:607e:4e7e:b3e7:36d:f7f3
 */
function ipv6() {
    return Array
        .from({ length: 8 })
        .map(() => basic_1.integer(0, 65535).toString(16))
        .join(':');
}
exports.ipv6 = ipv6;
/**
 * Returns a port number
 *
 * @example
 *
 * port()
 * // => 5286
 */
function port(min, max) {
    min = utils_1.clamp(utils_1.defaultTo(min, 0), 0, 65535);
    max = utils_1.clamp(utils_1.defaultTo(max, 65535), 0, 65535);
    return basic_1.integer(min, max);
}
exports.port = port;
/**
 * Returns a domain
 *
 * @example
 *
 * domain()
 * // => 'xfoekq.cn'
 *
 * domain('hk')
 * // => 'voewpzn.hk'
 */
function domain(_tld) {
    return text_1.word() + '.' + (_tld || tld());
}
exports.domain = domain;
/**
 * Returns a pathname
 *
 * @example
 * pathname()
 * // => '/o8qpgvk/ask2ih/cey'
 *
 * pathname(2)
 * // => '/qutmva/flq2'
 */
function pathname(length) {
    length = utils_1.defaultTo(length, basic_1.integer(1, 6));
    return '/' + Array
        .from({ length })
        .map(() => basic_1.string(constants_1.pools.letter + constants_1.pools.number, 1, 8))
        .join('/');
}
exports.pathname = pathname;
/**
 * Returns an url
 *
 * @example
 *
 * url()
 * // => 'mailto://wobiq.com/jqpvk/qogx'
 *
 * url('https')
 * // => 'https://ieyqjo.org/cktoqm/zlire/aw/gaoq'
 *
 * url('https', 'github.com')
 * // => 'https://github.com/owiytj/akwpt/bns'
 *
 * url('https', 'github.com', 'varHarrie')
 * // => 'https://github.com/varHarrie/nchgyuel/aj/qigpjvr'
 */
function url(_protocol, host, prefix) {
    _protocol = utils_1.defaultTo(_protocol, protocol());
    host = utils_1.defaultTo(host, domain());
    const path = utils_1.defaultTo(prefix, '') + pathname();
    return _protocol + '://' + host + path;
}
exports.url = url;
/**
 * Return a email address
 *
 * @example
 *
 * email()
 * // => 'rih.zqgll@owk.cn'
 *
 * email('gmail.com')
 * // => 'bqignwo.qzu@gmail.com'
 */
function email(_domain) {
    const head = Array
        .from({ length: basic_1.integer(1, 2) })
        .map(() => basic_1.string(constants_1.pools.username, 1, 8))
        .join('.');
    return head + '@' + (_domain || domain());
}
exports.email = email;
