"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const dayjs_1 = __importDefault(require("dayjs"));
const advancedFormat_1 = __importDefault(require("dayjs/plugin/advancedFormat"));
const constants_1 = require("./constants");
const generators_1 = require("./generators");
dayjs_1.default.extend(advancedFormat_1.default);
/**
 * Checks if `target` is a Class
 *
 * @example
 *
 * isClass(class Foo { })
 * // => true
 *
 * isClass(function foo () { })
 * // => false
 *
 * isClass({})
 * // => false
 */
function isClass(target) {
    return typeof target === 'function'
        && /^class\s/.test(Function.prototype.toString.call(target));
}
exports.isClass = isClass;
/**
 * Checks if `func` is a Function
 *
 * @example
 *
 * isFunction(function foo () { })
 * // => true
 *
 * isFunction(class Foo { })
 * // => false
 *
 * isFunction('')
 * // => false
 */
function isFunction(func) {
    return typeof func === 'function' && !isClass(func);
}
exports.isFunction = isFunction;
/**
 * Returns value that determined by original value
 *
 * @example
 *
 * defaultTo(1, 2)
 * // => 1
 *
 * defaultTo(undefined, 10)
 * // => 10
 *
 * defaultTo(NaN, 10)
 * // => 10
 *
 * defaultTo(null, 10)
 * // => 10
 */
function defaultTo(value, defaultValue) {
    return (value === undefined
        || value === null
        || typeof defaultValue === 'number' && isNaN(value))
        ? defaultValue
        : value;
}
exports.defaultTo = defaultTo;
/**
 * Returns value that determined by original value
 *
 * @example
 *
 * defaultBy(1, 2, 3)
 * // => 2
 *
 * defaultBy(undefined, 10, 20)
 * // => 20
 *
 * defaultBy(NaN, 10, 20)
 * // => 20
 *
 * defaultBy(null, 10, 20)
 * // => 20
 */
function defaultBy(value, trueValue, falseValue) {
    return (value === undefined || value === null || isNaN(value))
        ? falseValue : trueValue;
}
exports.defaultBy = defaultBy;
/**
 * Returns an integer from any type
 *
 * @example
 *
 * ensureInteger(12)
 * // => 12
 *
 * ensureInteger(2.5)
 * // => 2
 *
 * ensureInteger(null)
 * // => 0
 *
 * ensureInteger(null, 1)
 * // => 1
 */
function ensureInteger(value, defaultValue = 0) {
    const num = parseInt(value, 10);
    return isNaN(num) ? defaultValue : num;
}
exports.ensureInteger = ensureInteger;
/**
 * Returns a natural number from any type
 *
 * @example
 *
 * ensureNatural(12)
 * // => 12
 *
 * ensureNatural(2.5)
 * // => 2
 *
 * ensureNatural(-1)
 * // => 0
 *
 * ensureInteger(null)
 * // => 0
 */
function ensureNatural(value, defaultValue = 0) {
    const num = ensureInteger(value, defaultValue);
    return num < 0 ? 0 : num;
}
exports.ensureNatural = ensureNatural;
/**
 * Returns a number within lower and upper bounds
 *
 * @example
 *
 * clamp(5, 0, 10)
 * // => 5
 *
 * clamp(-3, 0, 10)
 * // => 0
 *
 * clamp(18, 0, 10)
 * // => 10
 */
function clamp(value, lower, upper) {
    const min = Math.min(lower, upper);
    const max = Math.max(lower, upper);
    return value < min ? min : value > max ? max : value;
}
exports.clamp = clamp;
/**
 * Returns a string with capital first letter
 *
 * @example
 *
 * capitalize('abc')
 * // => 'Abc'
 */
function capitalize(value) {
    return value.substring(0, 1).toUpperCase() + value.substring(1);
}
exports.capitalize = capitalize;
/**
 * Transform DateLike variable to string with format string
 *
 * @example
 *
 * formatDate('2018-12-06 12:00:00', 'YYYY/MM/DD HH:mm')
 * // => 2018/12/06 12:00
 *
 * formatDate(1544068800000, 'YYYY/MM/DD HH:mm')
 * // => 2018/12/06 12:00
 *
 * formatDate(new Date(), 'YYYY/MM/DD HH:mm')
 * // => 2018/12/06 12:00
 */
function formatDate(date, formatStr) {
    return dayjs_1.default(date).format(formatStr);
}
exports.formatDate = formatDate;
function randomDate(min, max) {
    const minDate = max && min ? new Date(min) : constants_1.MIN_DATE;
    const maxDate = max ? new Date(max) : min ? new Date(min) : constants_1.MAX_DATE;
    return new Date(generators_1.integer(minDate.getTime(), maxDate.getTime()));
}
exports.randomDate = randomDate;
/**
 * Checks if `obj` is a true object
 *
 * @example
 *
 * isTrueObject({foo: 'bar'})
 * // => true
 *
 * isTrueObject(new Object())
 * // => true
 *
 * isTrueObject(null)
 * // => false
 *
 * isTrueObject(function foo () { })
 * // => false
 */
function isTrueObject(obj) {
    return !!obj && typeof obj === 'object';
}
exports.isTrueObject = isTrueObject;
/**
 * Returns an object with properties picked from another
 *
 * @example
 *
 * pickProps({ name: 'Harrie', age: 18, gender: 'male' }, 'name age')
 * // => { name: 'Harrie', age: 18 }
 *
 * pickProps({ name: 'Harrie', age: 18, gender: 'male' }, ['name', 'age'])
 * // => { name: 'Harrie', age: 18 }
 */
function pickProps(source, props) {
    if (!isTrueObject(source))
        return {};
    if (!props.length)
        return source;
    props = Array.isArray(props) ? props : props.split(' ');
    return props.reduce((r, prop) => {
        r[prop] = source[prop];
        return r;
    }, {});
}
exports.pickProps = pickProps;
/**
 * Returns an array with items picked from another
 *
 * @example
 *
 * pickProps(['a', 'b', 'c', 'd'], 3)
 * // => ['d', 'a', 'c']
 */
function pickItems(list, length) {
    length = length < list.length ? length : list.length;
    return list
        .sort(() => Math.random() - 0.5)
        .slice(0, length);
}
exports.pickItems = pickItems;
/**
 * Converts hsl to rgb
 *
 * @reference
 *
 * https://www.w3.org/TR/css-color-3/#hsl-color
 *
 * @example
 *
 * hslToRgb(0, 1, 0.5)
 * // => [1, 0, 0]
 */
function hslToRgb(h, s, l) {
    const q = l < 0.5 ? l * (s + 1) : l + s - l * s;
    const p = l * 2 - q;
    return [h + 1 / 3, h, h - 1 / 3]
        .map((v) => {
        if (v < 0)
            v += 1;
        if (v > 1)
            v -= 1;
        if (v < 1 / 6)
            return p + (q - p) * 6 * v;
        if (v < 1 / 2)
            return q;
        if (v < 2 / 3)
            return p + (q - p) * (2 / 3 - v) * 6;
        return p;
    });
}
exports.hslToRgb = hslToRgb;
