export declare type Func<R = any> = (...args: any[]) => R;
/**
 * Checks if `target` is a Class
 *
 * @example
 *
 * isClass(class Foo { })
 * // => true
 *
 * isClass(function foo () { })
 * // => false
 *
 * isClass({})
 * // => false
 */
export declare function isClass(target: any): target is new (...args: any[]) => any;
/**
 * Checks if `func` is a Function
 *
 * @example
 *
 * isFunction(function foo () { })
 * // => true
 *
 * isFunction(class Foo { })
 * // => false
 *
 * isFunction('')
 * // => false
 */
export declare function isFunction(func: any): func is Func;
/**
 * Returns value that determined by original value
 *
 * @example
 *
 * defaultTo(1, 2)
 * // => 1
 *
 * defaultTo(undefined, 10)
 * // => 10
 *
 * defaultTo(NaN, 10)
 * // => 10
 *
 * defaultTo(null, 10)
 * // => 10
 */
export declare function defaultTo<T>(value: any, defaultValue: T): T;
/**
 * Returns value that determined by original value
 *
 * @example
 *
 * defaultBy(1, 2, 3)
 * // => 2
 *
 * defaultBy(undefined, 10, 20)
 * // => 20
 *
 * defaultBy(NaN, 10, 20)
 * // => 20
 *
 * defaultBy(null, 10, 20)
 * // => 20
 */
export declare function defaultBy<T>(value: any, trueValue: any, falseValue: any): any;
/**
 * Returns an integer from any type
 *
 * @example
 *
 * ensureInteger(12)
 * // => 12
 *
 * ensureInteger(2.5)
 * // => 2
 *
 * ensureInteger(null)
 * // => 0
 *
 * ensureInteger(null, 1)
 * // => 1
 */
export declare function ensureInteger(value: any, defaultValue?: number): number;
/**
 * Returns a natural number from any type
 *
 * @example
 *
 * ensureNatural(12)
 * // => 12
 *
 * ensureNatural(2.5)
 * // => 2
 *
 * ensureNatural(-1)
 * // => 0
 *
 * ensureInteger(null)
 * // => 0
 */
export declare function ensureNatural(value: any, defaultValue?: number): number;
/**
 * Returns a number within lower and upper bounds
 *
 * @example
 *
 * clamp(5, 0, 10)
 * // => 5
 *
 * clamp(-3, 0, 10)
 * // => 0
 *
 * clamp(18, 0, 10)
 * // => 10
 */
export declare function clamp(value: number, lower: number, upper: number): number;
/**
 * Returns a string with capital first letter
 *
 * @example
 *
 * capitalize('abc')
 * // => 'Abc'
 */
export declare function capitalize(value: string): string;
export declare type DateType = number | string | Date;
/**
 * Transform DateLike variable to string with format string
 *
 * @example
 *
 * formatDate('2018-12-06 12:00:00', 'YYYY/MM/DD HH:mm')
 * // => 2018/12/06 12:00
 *
 * formatDate(1544068800000, 'YYYY/MM/DD HH:mm')
 * // => 2018/12/06 12:00
 *
 * formatDate(new Date(), 'YYYY/MM/DD HH:mm')
 * // => 2018/12/06 12:00
 */
export declare function formatDate(date: DateType, formatStr: string): string;
/**
 * Return random Date object
 *
 * @example
 *
 * randomDate()
 * // => [Thu Dec 06 2018 12:00:00]
 *
 * randomDate('2018-12-06 00:00:00')
 * // => [Tue Jul 26 2016 17:32:58]
 *
 * randomDate('2018-12-05 00:00:00', '2018-12-06 00:00:00')
 * // => [Wed Dec 05 2018 13:27:48]
 */
export declare function randomDate(): Date;
export declare function randomDate(max: DateType): Date;
export declare function randomDate(min: DateType, max: DateType): Date;
/**
 * Checks if `obj` is a true object
 *
 * @example
 *
 * isTrueObject({foo: 'bar'})
 * // => true
 *
 * isTrueObject(new Object())
 * // => true
 *
 * isTrueObject(null)
 * // => false
 *
 * isTrueObject(function foo () { })
 * // => false
 */
export declare function isTrueObject(obj: any): obj is Object;
/**
 * Returns an object with properties picked from another
 *
 * @example
 *
 * pickProps({ name: 'Harrie', age: 18, gender: 'male' }, 'name age')
 * // => { name: 'Harrie', age: 18 }
 *
 * pickProps({ name: 'Harrie', age: 18, gender: 'male' }, ['name', 'age'])
 * // => { name: 'Harrie', age: 18 }
 */
export declare function pickProps<T extends Object>(source: T, props: string | string[]): Partial<T>;
/**
 * Returns an array with items picked from another
 *
 * @example
 *
 * pickProps(['a', 'b', 'c', 'd'], 3)
 * // => ['d', 'a', 'c']
 */
export declare function pickItems<T>(list: T[], length: number): T[];
/**
 * Converts hsl to rgb
 *
 * @reference
 *
 * https://www.w3.org/TR/css-color-3/#hsl-color
 *
 * @example
 *
 * hslToRgb(0, 1, 0.5)
 * // => [1, 0, 0]
 */
export declare function hslToRgb(h: number, s: number, l: number): number[];
